#ifndef __MATRIX_CPP
    #define __MATRIX_CPP
    
    
    template<class T> Matrix<T>::Matrix(int h, int w)
    {
        content = std::vector< std::vector<T> >(h,std::vector<T>(w));
    }
    
    template<class T> Matrix<T>::Matrix(int h, int w, T initial_value)
    {
        content = std::vector< std::vector<T> >(h,std::vector<T>(w, initial_value));
    }
    
    template<class T> Matrix<T>::Matrix(int h, int w, T function(int, int)) 
    {
        content = std::vector< std::vector<T> >(h,std::vector<T>(w));
        for(int i = 0; i < h; i++)
            for(int j = 0; j < w; j++)
                content[i][j] = function(i,j);
    }
    
    template<class T> Matrix<T>::Matrix(const Matrix<T>& pattern)
    {
        content = pattern.content;
    }
    
    template<class T> bool Matrix<T>::operator=(const Matrix<T>& pattern)
    {
        return content = pattern.content;
    }




    template<class T> std::vector<T>& Matrix<T>::operator[](int y) const
    {
        return content[y];
    }
    
    template<class T> int Matrix<T>::Height() const
    {
        return content.size();
    }
    
    template<class T> int Matrix<T>::Width() const
    {
        if(content.size() <= 0)
            return 0;
        return content[0].size();
    }
    
    template<class T> std::vector<T> Matrix<T>::Row(int row) const
    {
        return content[row];
    }
    
    template<class T> std::vector<T> Matrix<T>::Column(int column) const 
    {
        std::vector<T> result(Height());
        for(int i = Height()-1; i >= 0; --i)
            result[i] = content[i][column];
        return(result);
    }





    template<class T> Matrix<T> Matrix<T>::operator*(const Matrix<T>& X) const
    {
        if(Width() != X.Height())
            throw("Matrix.h: template<class T> Matrix<T> Matrix<T>::operator*(const Matrix<T>& X) const - Matrices incompatible - Width() must equal X.Height()!");
        
        Matrix<T> result(Height(),X.Width());
        for(int i = Height()-1; i >= 0; --i)
            for(int j = X.Width()-1; j >= 0; --j)
            {
                result.content[i][j] = content[i][0] * X.content[0][j];
                for(int k = Width(); k > 0; --k)
                    result.content[i][j] += content[i][k]  *  X.content[k][j];
            }
            
        return(result);
    }
    
    template<class T> Matrix<T> Matrix<T>::operator+(const Matrix<T>& X) const
    {
        if(Height() != X.Height() || Width() != X.Width())
            throw("Matrix.h: template<class T> Matrix<T> Matrix<T>::operator+(const Matrix<T>& X) const - Matrices can only be added if they have the same height and width");
        
        Matrix<T> result(Height(),Width());
        for(int y = Height()-1; y >= 0; --y)
            for(int x = Width()-1; x >= 0; --x)
                result.content[y][x] = content[y][x] + X.content[y][x];
            
        return(result);
    }
    
    template<class T> Matrix<T> Matrix<T>::operator-(const Matrix<T>& X) const
    {
        if(Height() != X.Height() || Width() != X.Width())
            throw("Matrix.h: template<class T> Matrix<T> Matrix<T>::operator-(const Matrix<T>& X) const - Matrices can only be subtracted if they have the same height and width");
        
        Matrix<T> result(Height(),Width());
        for(int y = Height()-1; y >= 0; --y)
            for(int x = Width()-1; x >= 0; --x)
                result.content[y][x] = content[y][x] - X.content[y][x];
            
        return(result);
    }
    
    template<class T> Matrix<T> Matrix<T>::operator*(const T& X) const
    {
        Matrix<T> result(Height(),Width());
        
        for(int x = Width()-1; x >= 0; --x)
            for(int y = Height()-1; y >= 0; --y)
                result.content[y][x] = content[y][x] * X;
            
        return(result);
    }
    
    template<class T> Matrix<T> operator*(const T& X, const Matrix<T>& M)
    {
        return M*X;
    }
    
    template<class T> Matrix<T> Matrix<T>::Transposed() const 
    {
        Matrix<T> result(Width(), Height());
        
        for(int i = Width()-1; i >= 0; --i)
            for(int j = Height()-1; j >= 0; --j)
                result.content[i][j] = content[j][i];
            
        return(result);
    }
    
    template<class T> Matrix<T> Matrix<T>::TensorProduct(const Matrix<T>& X) const
    {
        Matrix<T> result(Height()*X.Height(),Width()*X.Width());
        
        for(int y = Height()-1; y >= 0; --y)
            for(int x = Width()-1; x >= 0; --x)
                result.Overwrite(y*X.Height(), x*X.Width(), X*content[y][x]);
            
        return(result);
    }
    
    template<class T> Matrix<T> Matrix<T>::DirectSum(const Matrix<T>& X) const
    {
        Matrix<T> result(Height()+X.Height(), Width()+X.Width(), T(0));
        result.Overwrite(0,0,*this);
        result.Overwrite(Height(),Width(),X);
        return(result);
    }




    template<class T> bool Matrix<T>::AddRow(int row, int to_row, T factor)
    {
        for(int i = Width()-1; i >= 0; --i)
            content[to_row][i] += factor*content[row][i];
    }
    
    template<class T> bool Matrix<T>::AddColumn(int column, int to_column, T factor)
    {
        for(int i = Height()-1; i >= 0; --i)
            content[i][to_column] += factor*content[i][column];
    }
    
    template<class T> void Matrix<T>::SwapRows(int row1, int row2)
    {
        content.swap(row1,row2);
    }
    
    template<class T> void Matrix<T>::SwapColumns(int column1, int column2)
    {
        for(int i = Height()-1; i >= 0; --i)
            content[i].swap(column1,column2);
    }
    
    template<class T> void Matrix<T>::MultiplyRow(int row, T factor)
    {
        for(int i = Width()-1; i >= 0; --i)
            content[row][i] *= factor;
    }
    
    template<class T> void Matrix<T>::MultiplyColumn(int column, T factor)
    {
        for(int i = Height()-1; i >= 0; --i)
            content[i][column] *= factor;
    }
    
    template<class T> void Matrix<T>::PivotiseRow(int row, int column)
    {
        for(int i = Height()-1; i >= 0; --i)
            if(i != row)
                AddRow(row,i,T(0)-content[i][column]/content[row][column]);
    }
    
    template<class T> void Matrix<T>::PivotiseColumn(int row, int column)
    {
        for(int i = Width()-1; i >= 0; --i)
            if(i != column)
                AddColumn(column,i,T(0)-(content[row][i]/content[row][column]));
    }





    template<class T> bool Matrix<T>::InsertRow(int pos, int count)
    {
        return InsertRows(pos, Matrix<T>(count,Width()));
    }
    
    template<class T> bool Matrix<T>::InsertRows(int pos, const Matrix<T>& which)
    {
        if(Width() != which.Width())
            return(false);
        return( content.InsertElements(pos, which.content) );
    }
    
    template<class T> bool Matrix<T>::InsertColumn(int pos, int count)
    {
        return InsertColumns(pos, Matrix<T>(Height(),count));
    }
    
    template<class T> bool Matrix<T>::InsertColumns(int pos, const Matrix<T>& which)
    {
        if(Height() != which.Height())
            return(false);
        bool success = true;
        for(int i = Height()-1; i >= 0; --i)
            success = content[i].InsertElements(pos,which.content[i]) && success;
        return(success);
    }
    
    template<class T> void Matrix<T>::DeleteRow(int pos)
    {
        content.DeleteElement(pos);
    }
    
    template<class T> void Matrix<T>::DeleteColumn(int pos)
    {
        for(int i = Height(); i >= 0; --i)
            content[i].DeleteElement(pos);
    }
    
    template<class T> void Matrix<T>::DeleteRows(int pos, int count)
    {
        content.DeleteElements(pos, count);
    }
    
    template<class T> void Matrix<T>::DeleteColumns(int pos, int count)
    {
        for(int i = Height(); i >= 0; --i)
            content[i].DeleteElements(pos, count);
    }
    
    template<class T> void Matrix<T>::DeleteRows(const std::vector<bool>& map)
    {
        content.DeleteElements(map);
    }
    
    template<class T> void Matrix<T>::DeleteColumns(const std::vector<bool>& map)
    {
        for(int i = Height(); i >= 0; --i)
            content[i].DeleteElements(map);
    }
    
    template<class T> void Matrix<T>::Overwrite(int StartY, int StartX, const Matrix<T>& newvalues) 
    {
        int minh = StartY+newvalues.Height() > Height() ? Height() : StartY+newvalues.Height();
        int minw = StartX+newvalues.Width() > Width() ? Width() : StartX+newvalues.Width();
        for(int i = StartY, j=0; i < minh; ++i, ++j)
            for(int a = StartX, b=0; a < minw; ++a, ++b)
                content[i][a] = newvalues.content[j][b];
    }


    template<class T> ostream& operator<<(ostream& os, const Matrix<T>& M)
    {
        int h = M.Height(), w = M.Width();
        os << h << '\n';
        os << w << '\n';
        for(int i = 0; i < h; ++i)
        {
            for(int j = 0; j < w; ++j)
                os << M.content[i][j] << ' ';
            os << '\n';
        }
        return(os);
    }
    
    template<class T> istream& operator>>(istream& is, Matrix<T>& M)
    {
        int h, w;
        is >> h;
        is >> w;
        Matrix<T> temp(h,w);
        for(int i = 0; i < h; ++i)
            for(int j = 0; j < w; ++j)
                is >> temp.content[i][j];
        M = temp;
        return(is);
    }

#endif
