
#ifndef __MATRIX_H
    #define __MATRIX_H

    #include<iostream>
    #include<sstream>
    #include<vector>

    template<class T> class Matrix
    {
        template<class U> friend ostream& operator<<(ostream& os, const Matrix<U>& M);
        template<class U> friend istream& operator>>(istream& is, Matrix<U>& M);
        protected:
            std::vector< std::vector<T> > content;
            
        public:
            Matrix(int h, int w);                                         // system
            Matrix(int h, int w, T initial_value);
            Matrix(int h, int w, T function(int, int));
            Matrix(const Matrix& pattern);
            bool operator=(const Matrix<T>& pattern);


            std::vector<T>& operator[](int y) const;                      // reading
            int Height() const;
            int Width() const;
            std::vector<T> Row(int row) const;
            std::vector<T> Column(int column) const;


            Matrix<T> operator*(const Matrix<T>& X) const;                // default actions
            Matrix<T> operator+(const Matrix<T>& X) const;
            Matrix<T> operator-(const Matrix<T>& X) const;
            Matrix<T> operator*(const T& X) const;
            Matrix<T> Transposed() const;
            Matrix<T> TensorProduct(const Matrix<T>& X) const;
            Matrix<T> DirectSum(const Matrix<T>& X) const;


            bool AddRow(int row, int to_row, T factor);                   // Elementary operations
            bool AddColumn(int column, int to_row, T factor);
            void SwapRows(int row1, int row2);
            void SwapColumns(int column1, int column2);
            void MultiplyRow(int row, T factor);
            void MultiplyColumn(int column, T factor);
            void PivotiseRow(int row, int column);
            void PivotiseColumn(int row, int column);


            bool InsertRow(int pos, int count=1);                         // manipulations
            bool InsertRows(int pos, const Matrix<T>&);
            bool InsertColumn(int pos, int count=1);
            bool InsertColumns(int pos, const Matrix<T>&);
            void DeleteRow(int pos);
            void DeleteColumn(int pos);
            void DeleteRows(int pos, int count=1);
            void DeleteColumns(int pos, int count=1);
            void DeleteRows(const std::vector<bool>& map);
            void DeleteColumns(const std::vector<bool>& map);
            void Overwrite(int StartY, int StartX, const Matrix<T>& newvalues);
    };

    template<class T> ostream& operator<<(ostream& os, const Matrix<T>& M);
    template<class T> istream& operator>>(istream& is, Matrix<T>& M);
    template<class T> Matrix<T> operator*(const T& X, const Matrix<T>& M);

    #include "matrix.cpp"

#endif

